unit fReportPreView;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ImgList, Menus, ActnList, StdCtrls, ComCtrls, ToolWin, 
  RpBase, RpFiler, RpRenderPrinter, RpRenderPreview,
  RpDefine, RpSystem, RpFormSetup, RpRender, RpRenderCanvas;

type
  TfrmReportPreView = class(TForm)
    ScrollBox1: TScrollBox;
    sbarMain: TStatusBar;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    Panel1: TPanel;
    PageEdit: TEdit;
    PageLabel: TPanel;
    ToolButton10: TToolButton;
    ToolButton11: TToolButton;
    ToolButton12: TToolButton;
    ToolButton13: TToolButton;
    ToolButton14: TToolButton;
    Panel3: TPanel;
    ZoomEdit: TEdit;
    Panel2: TPanel;
    ToolButton9: TToolButton;
    ToolButton15: TToolButton;
    ActionList1: TActionList;
    actnFile_Print: TAction;
    actnFile_Save: TAction;
    actnFile_Exit: TAction;
    actnFile_Open: TAction;
    actnPage_First: TAction;
    actnPage_Next: TAction;
    actnPage_Previous: TAction;
    actnPage_Last: TAction;
    actnZoom_In: TAction;
    actnZoom_Out: TAction;
    actnZoom_PageWidth: TAction;
    actnZoom_Page: TAction;
    actnHelp_About: TAction;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    SaveAs1: TMenuItem;
    Print1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Page1: TMenuItem;
    First1: TMenuItem;
    Previous1: TMenuItem;
    Next1: TMenuItem;
    Last1: TMenuItem;
    N2: TMenuItem;
    GotoPage1: TMenuItem;
    Zoom1: TMenuItem;
    In1: TMenuItem;
    Out1: TMenuItem;
    N3: TMenuItem;
    Page2: TMenuItem;
    PageWidth1: TMenuItem;
    PopupMenu1: TPopupMenu;
    In2: TMenuItem;
    Out2: TMenuItem;
    ilstActions: TImageList;
    dlogOpen: TOpenDialog;
    dlogSave: TSaveDialog;
    PreviewTimer: TTimer;
    actnPage_GoTo: TAction;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure NDRPreviewPageChange(Sender: TObject);
    procedure actnPage_FirstExecute(Sender: TObject);
    procedure actnPage_PreviousExecute(Sender: TObject);
    procedure actnPage_NextExecute(Sender: TObject);
    procedure actnPage_LastExecute(Sender: TObject);
    procedure actnFile_ExitExecute(Sender: TObject);
    procedure actnZoom_InExecute(Sender: TObject);
    procedure actnZoom_OutExecute(Sender: TObject);
    procedure actnFile_PrintExecute(Sender: TObject);
    procedure actnFile_OpenExecute(Sender: TObject);
    procedure actnFile_SaveExecute(Sender: TObject);
    procedure actnZoom_PageExecute(Sender: TObject);
    procedure actnZoom_PageWidthExecute(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure actnPage_GoToExecute(Sender: TObject);
  private
    FPageNum: integer;
    FOnGenerate: TNotifyEvent;
    FInputFileName: String;
    FInputStream: TStream;
    FOnAfterPreviewPrint: TNotifyEvent;

    Procedure NewRenderPage(Sender: TObject);
  public
    ReportSystem: TRvSystem;
    FormClosed: boolean;
    RvRenderPrinter: TRvRenderPrinter;
    RvRenderPreview: TRvRenderPreview;
    //
    Procedure InitFromRPSystem;
    procedure OpenFile(const AFileName: string);
    Class Procedure PreviewFile(Const AFileName: String);

    Property PageNum: integer read FPageNum write FPageNum;
    Property OnGenerate: TNotifyEvent read FOnGenerate write FOnGenerate;
    Property OnAfterPreviewPrint: TNotifyEvent read FOnAfterPreviewPrint write FOnAfterPreviewPrint;
    Property InputFileName: String read FInputFileName write FInputFileName;
    Property InputStream: TStream read FInputStream write FInputStream;
  end;

var
  frmReportPreView: TfrmReportPreView;

implementation

uses 
  fMain;

{$R *.dfm}

procedure TfrmReportPreView.FormCreate(Sender: TObject);
begin
  FPageNum := 1;

  RvRenderPreview := TRvRenderPreview.Create(self);
  With RvRenderPreview Do Begin
    OnPageChange := NDRPreviewPageChange;
    ShadowDepth := 2;
    MarginPercent := 5;
    ZoomFactor := 100;
    ScrollBox := ScrollBox1;
  End; 

  RvRenderPrinter := TRvRenderPrinter.Create(self);
  With RvRenderPrinter Do Begin
  End; 
end;

procedure TfrmReportPreView.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  FormClosed := true;
end;

procedure TfrmReportPreView.FormDestroy(Sender: TObject);
begin
  FreeAndNil(RvRenderPreview);
  FreeAndNil(RvRenderPrinter);
end;

procedure TfrmReportPreView.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
Var
  I1: integer;
Begin
  Case Key Of
    vk_Up: Begin
        If (Shift = []) Or (Shift = [ssCtrl]) Then Begin
          With ScrollBox1.VertScrollBar Do Begin
            I1 := Position - Increment;
            If I1 < 0 Then Begin
              If RvRenderPreview.CurrentPage > 1 Then Begin
                RvRenderPreview.PrevPage;
                I1 := Range;
              End Else Begin
                I1 := 0;
              End; { else }
            End; { if }
            Position := I1;
            Key := 0;
          End; { with }
        End; { if }
      End;
    vk_Down: Begin
        If (Shift = []) Or (Shift = [ssCtrl]) Then Begin
          With ScrollBox1.VertScrollBar Do Begin
            I1 := Position + Increment;
            If I1 >= Range Then Begin
              I1 := Range - 1;
            End; { if }
            Position := I1;
            If (I1 > Position) And (RvRenderPreview.CurrentPage < RvRenderPreview.Pages) Then Begin
              RvRenderPreview.NextPage;
              Position := 0;
            End; { if }
            Key := 0;
          End; { with }
        End; { if }
      End;
    vk_Home: Begin
        If (Shift = []) Or (Shift = [ssCtrl]) Then Begin
          With ScrollBox1.VertScrollBar Do Begin
            Position := 0;
            Key := 0;
          End; { with }
        End; { if }
      End;
    vk_End: Begin
        If (Shift = []) Or (Shift = [ssCtrl]) Then Begin
          With ScrollBox1.VertScrollBar Do Begin
            Position := Range - 1;
            Key := 0;
          End; { with }
        End; { if }
      End;
    vk_Prior: Begin
        If (Shift = []) Or (Shift = [ssCtrl]) Then Begin
          With ScrollBox1.VertScrollBar Do Begin
            If (Position = 0) And (RvRenderPreview.CurrentPage > 1) Then Begin
              RvRenderPreview.PrevPage;
              I1 := Range - 1;
            End Else Begin
              I1 := Position - ScrollBox1.Height;
              If I1 < 0 Then Begin
                I1 := 0;
              End; { if }
            End; { else }
            Position := I1;
            Key := 0;
          End; { with }
        End; { if }
      End;
    vk_Next: Begin
        If (Shift = []) Or (Shift = [ssCtrl]) Then Begin
          With ScrollBox1.VertScrollBar Do Begin
            I1 := Position + ScrollBox1.Height;
            If I1 >= Range Then Begin
              If RvRenderPreview.CurrentPage < RvRenderPreview.Pages Then Begin
                RvRenderPreview.NextPage;
                I1 := 0;
              End Else Begin
                I1 := Range - 1;
              End; { else }
            End; { if }
            Position := I1;
            Key := 0;
          End; { with }
        End; { if }
      End;
    vk_Left: Begin
        With ScrollBox1.HorzScrollBar Do Begin
          If Shift = [ssCtrl] Then Begin
            I1 := Position - Increment;
            If I1 < 0 Then Begin
              I1 := 0;
            End; { if }
            Position := I1;
            Key := 0;
          End Else If Shift = [ssCtrl, ssShift] Then Begin
            I1 := Position - ScrollBox1.Width;
            If I1 < 0 Then Begin
              I1 := 0;
            End; { if }
            Position := I1;
            Key := 0;
          End; { else }
        End; { with }
      End;
    vk_Right: Begin
        With ScrollBox1.HorzScrollBar Do Begin
          If Shift = [ssCtrl] Then Begin
            I1 := Position + Increment;
            If I1 >= Range Then Begin
              I1 := Range - 1;
            End; { if }
            Position := I1;
            Key := 0;
          End Else If Shift = [ssCtrl, ssShift] Then Begin
            I1 := Position + ScrollBox1.Width;
            If I1 >= Range Then Begin
              I1 := Range - 1;
            End; { if }
            Position := I1;
            Key := 0;
          End; { else }
        End; { with }
      End;
  End; { case }
end;

procedure TfrmReportPreView.FormKeyPress(Sender: TObject; var Key: Char);
Var
  ValidKeys: String;
Begin
  If (Key = #27) Or (Key = 'x') Then Begin
    Close;
    Key := #0;
  End Else Begin
    ValidKeys := {Trans-} 'eEwWfFnNpP+-01234567890.'#13#8;
    If Pos(Key, ValidKeys) > 0 Then Begin
      Case Key Of
        'e', 'E': Begin
            RvRenderPreview.RenderPage(RvRenderPreview.Pages);
            Key := #0;
          End;
        'n', 'N': Begin
            If RvRenderPreview.CurrentPage < RvRenderPreview.Pages Then Begin
              RvRenderPreview.NextPage;
            End; { if }
            Key := #0;
          End;
        'p', 'P': Begin
            If RvRenderPreview.CurrentPage > 1 Then Begin
              RvRenderPreview.PrevPage;
            End; { if }
            Key := #0;
          End;
      End; { case }
    End Else Begin
      Key := #0;
    End; { else }
  End; { else }
end;

procedure TfrmReportPreView.InitFromRPSystem;
begin
  If InputFileName <> '' Then Begin
    RvRenderPreview.NDRStream := TFileStream.Create(InputFileName, fmOpenRead Or fmShareDenyWrite);
  End Else Begin
    RvRenderPreview.NDRStream := InputStream;
  End; { else }
  RvRenderPreview.Render;
  ReportSystem.PreviewShowEvent(RvRenderPreview);
end;

procedure TfrmReportPreView.NewRenderPage(Sender: TObject);
begin
// Update page count
  PageEdit.Text := Trans('of') + ' ' + IntToStr((Sender As TRvNDRWriter).JobPages);
// Make sure everything is refreshed
  Application.ProcessMessages;
end;

procedure TfrmReportPreView.OpenFile(const AFileName: string);
begin
  with RvRenderPreview do begin
    InputFileName := AFileName;
    NDRStream.Free;
    Render(InputFileName);
    Caption := Application.Title + {Trans-}' - [ ' + ExtractFileName(InputFileName) + {Trans-}' ]';
  end; 
end;

class procedure TfrmReportPreView.PreviewFile(const AFileName: String);
begin
  With TfrmReportPreView.Create(Application) Do Begin
    OpenFile(AFileName);
    Visible := true;
  End; 
end;

procedure TfrmReportPreView.NDRPreviewPageChange(Sender: TObject);
begin
  with RvRenderPreview do begin
    PageEdit.Text:= IntToStr(CurrentPage);
  end; 
end;

procedure TfrmReportPreView.actnPage_FirstExecute(Sender: TObject);
begin   //
  RvRenderPreview.RenderPage(1);
end;

procedure TfrmReportPreView.actnPage_PreviousExecute(Sender: TObject);
begin  //e
  RvRenderPreview.PrevPage;
end;

procedure TfrmReportPreView.actnPage_NextExecute(Sender: TObject);
begin  //
  RvRenderPreview.NextPage;
end;

procedure TfrmReportPreView.actnPage_LastExecute(Sender: TObject);
begin  //
  RvRenderPreview.RenderPage(RvRenderPreview.Pages);
end;

procedure TfrmReportPreView.actnPage_GoToExecute(Sender: TObject);
var
  sInputStr: string;
  iPageNum: integer;
begin  //
  sInputStr:= InputBox('J', 'пJG    ', '1');
  TryStrtoint(sInputStr, iPageNum);
  if iPageNum<>-1 then begin
    RvRenderPreview.RenderPage(iPageNum);
  end;
end;

procedure TfrmReportPreView.actnFile_ExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TfrmReportPreView.actnZoom_InExecute(Sender: TObject);
begin  //j
  RvRenderPreview.ZoomIn;
end;

procedure TfrmReportPreView.actnZoom_OutExecute(Sender: TObject);
begin  //Yp
  RvRenderPreview.ZoomOut;
end;

procedure TfrmReportPreView.actnZoom_PageExecute(Sender: TObject);
begin  //㭶
  RvRenderPreview.ZoomFactor := RvRenderPreview.ZoomPageFactor;
end;

procedure TfrmReportPreView.actnZoom_PageWidthExecute(Sender: TObject);
begin  //e
  RvRenderPreview.ZoomFactor := RvRenderPreview.ZoomPageWidthFactor;
end;

procedure TfrmReportPreView.actnFile_PrintExecute(Sender: TObject);
begin  //CL
  ReportSystem.DefaultDest := rdPrinter;  
  ReportSystem.SystemSetups := [ssAllowCopies, ssAllowCollate, ssAllowDuplex, ssAllowDestPrinter];
  ReportSystem.OutputFileName := '';
  ReportSystem.SystemPrinter.Copies := 1;
  ReportSystem.SystemPrinter.FCollate := false;
  ReportSystem.SystemPrinter.FDuplex := dupSimplex;
  frmMain.RvProject1.Execute;
  ReportSystem.DefaultDest := rdPreview;
  ReportSystem.SystemSetups := [ssAllowSetup,ssAllowCopies,ssAllowCollate,ssAllowDuplex,ssAllowDestPreview,ssAllowDestPrinter,ssAllowDestFile,ssAllowPrinterSetup,ssAllowPreviewSetup];  
end;

procedure TfrmReportPreView.actnFile_OpenExecute(Sender: TObject);
begin
  if dlogOpen.Execute then
    OpenFile(dlogOpen.FileName);
end;

procedure TfrmReportPreView.actnFile_SaveExecute(Sender: TObject);
begin  //s
  if dlogSave.Execute then begin
    ReportSystem.SystemSetups := [ssAllowDestFile];
    ReportSystem.DefaultDest := rdFile;
    ReportSystem.OutputFileName := dlogSave.FileName;    
    ReportSystem.DoNativeOutput := false;
//    ReportSystem.RenderObject := nil;
    frmMain.RvProject1.Execute;
    ReportSystem.DefaultDest := rdPreview;
    ReportSystem.SystemSetups := [ssAllowSetup,ssAllowCopies,ssAllowCollate,ssAllowDuplex,ssAllowDestPreview,ssAllowDestPrinter,ssAllowDestFile,ssAllowPrinterSetup,ssAllowPreviewSetup];
  end;
end;


procedure TfrmReportPreView.FormShow(Sender: TObject);
begin  
  //
  PageLabel.Caption :=  ' of ' + IntToStr(RvRenderPreview.Pages);
end;


end.
